-- Copyright 2008 Steven Barth <steven@midlink.org>
-- Copyright 2011 Jo-Philipp Wich <jow@openwrt.org>
-- Licensed to the public under the Apache License 2.0.

local fs = require "nixio.fs"

config_xnet_dlms_master_client = Map("dlms-master", translate("xNet DLMS Master/ Client"),
    translate("Configure DLMS Master/ Client settings"))

-- Global section to enable/ disable config value
-------------------------------------------------
section_global = config_xnet_dlms_master_client:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
    translate("Enable DLMS Master/ Client"))
option_enabled.rmempty = false
device_id = section_global:option(Value, "device_id", translate("Device Id"),
    translate("Device Id to be sent to portal, usually MAC Id of the device"))

-- Polling Section
----------------------
section_polling = config_xnet_dlms_master_client:section(TypedSection, "polling", translate("DLMS Polling Configuration"))
section_polling.anonymous = true
section_polling.addremove = true
section_polling.template = "cbi/tblsection"

--id
polling_id = section_polling:option(Value, "id", translate("Name of this polling config"),
    translate("Choose a name for this configuration. Cannot contain space or special characters except _."))

logical_name = section_polling:option(Value, "logical_name", translate("Logical Name"),
    translate("Enter logical name of the DLMS profile in dot notation e.g. 1.0.254.1.240"))

read_mode = section_polling:option(ListValue, "read_mode", translate("Read Mode"),
    translate("DLMS Profiles can be read by Entry number in the table or by time duration range. Load profile/ Historical data is usually read by duration range, instantaneous profile are usually read by entry number"))
read_mode.default = "by_value"
read_mode:value("by_entry", translate("By Entry/ Entry counter"))
read_mode:value("by_range", translate("By Range/ Duration"))

no_of_days = section_polling:option(ListValue, "no_of_days", translate("Number of days / Duration for Load Profile"),
    translate("Select number of days to read the historical data (Only for Load Profile)"))
no_of_days.datatype = "uinteger"
no_of_days.default = "1"
no_of_days:value("1", translate("Last 1 day"))
no_of_days:value("7", translate("Last 7 days"))
no_of_days:value("30", translate("Last 30 days"))
no_of_days:depends("read_mode", "by_range")

polling_frequency = section_polling:option(Value, "polling_frequency", translate("Polling frequency"),
    translate("Enter value in seconds. Enter 0 to read once upon connection to the meter"))
polling_frequency.datatype = "uinteger"


-- Connection Section
----------------------
section_connection = config_xnet_dlms_master_client:section(TypedSection, "connection", translate("DLMS Connections"))
section_connection.anonymous = true
section_connection.addremove = true

--id
connection_id = section_connection:option(Value, "id", translate("Name of the connection"),
    translate("Choose a name for this connection. Cannot contain space or special characters except _."))

--connection
connection = section_connection:option(ListValue, "connection", translate("Connection Type"),
    translate("Select Serial port or DLSM over TCP"))
connection.default = "serial"
connection:value("serial", translate("Serial Port"))
connection:value("tcp", translate("TCP/IP"))

--connection
serial_port = section_connection:option(ListValue, "serial_port", translate("Serial Port"))
serial_port.default = "/dev/ttyXR0"
serial_port:value("/dev/ttyS0", translate("COM1"))
serial_port:value("/dev/ttyXR0", translate("COM2"))
serial_port:value("/dev/ttyXR1", translate("COM3"))
serial_port:value("/dev/ttyXR2", translate("COM4"))
serial_port:value("/dev/ttyXR3", translate("COM5"))
serial_port:value("/dev/ttyUSB0", translate("USB1"))
serial_port:depends("connection", "serial")

--baud
baud = section_connection:option(ListValue, "baud", translate("Baud rate"))
baud.default = "9600"
baud:value("300", translate("300"))
baud:value("600", translate("600"))
baud:value("1200", translate("1200"))
baud:value("2400", translate("2400"))
baud:value("4800", translate("4800"))
baud:value("9600", translate("9600"))
baud:value("19200", translate("19200"))
baud:value("38400", translate("38400"))
baud:value("19200", translate("19200"))
baud:value("57600", translate("57600"))
baud:value("115200", translate("115200"))
baud:depends("connection", "serial")

--data_bit
data_bit = section_connection:option(ListValue, "data_bit", translate("Data Bits"))
data_bit.default = "8"
data_bit:value("7", translate("7 bits"))
data_bit:value("8", translate("8 bits"))
data_bit:value("9", translate("9 bits"))
data_bit:depends("connection", "serial")

--stop bits
parity = section_connection:option(ListValue, "parity", translate("Parity"))
parity.default = "N"
parity:value("N", translate("None"))
parity:value("E", translate("Even"))
parity:value("O", translate("Odd"))
parity:depends("connection", "serial")

--stop bits
stop_bit = section_connection:option(ListValue, "stop_bit", translate("Parity"))
stop_bit.default = "1"
stop_bit:value("0", translate("No stop bits"))
stop_bit:value("1", translate("1 stop bit"))
stop_bit:value("2", translate("2 stop bits"))
stop_bit:depends("connection", "serial")

--host
host = section_connection:option(Value, "host", translate("Host IP Address"), translate("IP Address of DLMS Meter"))
host.datatype = "ip4addr"
host.default = "192.168.1.254"
host:depends("connection", "tcp")

--port
port = section_connection:option(Value, "port", translate("Port"))
port.default = "4059"
port:depends("connection", "tcp")

--IEC
iec = section_connection:option(Flag, "iec", translate("DLMS/ IEC"),
    translate("Is this connection uses DLMS or IEC standard."))
iec.default = "0"
iec.rmempty = false

--HDLC Wrapper
hdlc_wrapper = section_connection:option(Flag, "hdlc_wrapper", translate("HDLC Wrapper"),
    translate("Is this TCP connection wrapper over HDLC. Usually used with Serial to TCP converters."))
hdlc_wrapper.default = "1"
hdlc_wrapper.rmempty = false
hdlc_wrapper:depends("connection", "tcp")


use_ln_ref = section_connection:option(ListValue, "use_ln_ref", translate("Use Logical/Short Name"))
use_ln_ref.default = "1"
use_ln_ref:value("1", translate("Use Logical Name Refrencing"))
use_ln_ref:value("0", translate("Use Short Name Refrencing"))


trace = section_connection:option(ListValue, "trace", translate("Trace"), translate("Trace messages"))
trace.default = "Verbose"
trace:value("Error", translate("Error"))
trace:value("Warning", translate("Warning"))
trace:value("Info", translate("Info"))
trace:value("Verbose", translate("Verbose"))
trace:value("Off", translate("Off"))


password = section_connection:option(Value, "password", translate("Password for authentication"))
password.datatype = "string"

security_type = section_connection:option(ListValue, "security_type", translate("Security Level"))
security_type.default = "None"
security_type:value("None", translate("None"))
security_type:value("Authentication", translate("Authentication"))
security_type:value("Encryption", translate("Encryption"))
security_type:value("AuthenticationEncryption", translate("AuthenticationEncryption"))


authentication = section_connection:option(ListValue, "authentication", translate("Authentication"))
authentication.default = "None"
authentication:value("None", translate("None"))
authentication:value("Low", translate("Low"))
authentication:value("High", translate("High"))
authentication:value("HighMd5", translate("HighMd5"))
authentication:value("HighSha1", translate("HighSha1"))
authentication:value("HighGmac", translate("HighGmac"))
authentication:value("HighSha256", translate("HighSha256"))


client_address = section_connection:option(Value, "client_address", translate("Client address.(Default: 16)"))
client_address.datatype = "uinteger"
client_address.default = "16"

server_address = section_connection:option(Value, "server_address", translate("Server address.(Default: 1)"))
server_address.datatype = "uinteger"
server_address.default = "1"

return config_xnet_dlms_master_client
