local fs = require "nixio.fs"

m = Map("dlms-to-mqtt", translate("DLMS to MQTT"))

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
    translate("Enable DLMS to MQTT"))
option_enabled.rmempty = false
option_enabled.default = "1"

s = m:section(TypedSection, "mqtt", translate("DLMS MQTT"), "")
s.anonymous = true
s.addremove = false

-- MQTT Section
----------------------
section_mqtt = m:section(TypedSection, "mqtt", translate("DLMS to MQTT IoT Configuration"))
section_mqtt.anonymous = true
section_mqtt.addremove = false

section_mqtt:tab("general", translate("General Configuration"))
section_mqtt:tab("advanced", translate("Advanced Configuration"))


mqtt_host = section_mqtt:taboption("general", Value, "host", translate("MQTT Server"), translate("MQTT Server IP address/ host name"))
mqtt_host.datatype = "host"
mqtt_host.default = "127.0.0.1"

mqtt_port = section_mqtt:taboption("general", Value, "port", "Port", translate("MQTT Server port number"))
mqtt_port.default = "1883"
mqtt_port.datatype = "port"

mqtt_username = section_mqtt:taboption("general", Value, "username", "Username", translate("MQTT client username (leave empty if anonymous)"))

mqtt_password = section_mqtt:taboption("general", Value, "password", "Password", translate("MQTT client password (leave empty if anonymous)"))
mqtt_password.password = true

mqtt_topic = section_mqtt:taboption("general", Value, "data_topic", translate("Topic"), translate("Topic to send DLMS Data"))

clean_session = section_mqtt:taboption("general", Flag, "clean_session", translate("Clean Session"),
    translate("Tells MQTT Broker if it should discard existing session."))
clean_session.default = "0"
clean_session.rmempty = false

qos = s:option(Value, "qos", "QoS (Quality Of Service)")
qos.datatype = "range(0,2)"
qos.default = "1"

retain = s:option(ListValue, "retain", "Retain", translate("Makes the message retained."))
retain:value("1", translate("Yes"))
retain:value("0", translate("No"))
retain.default = "0"

client_id = section_mqtt:taboption("general", Value, "client_id", "Client Id", translate("Leave blank to generate random client id on connection"))

will_topic = section_mqtt:taboption("advanced", Value, "will_topic", translate("Will Topic"))
will_topic.datatype = "range(0,2)"

will_payload = section_mqtt:taboption("advanced", Value, "will_payload", "Will Payload", translate("Payload to publish in case connection is interrupted"))

will_qos = section_mqtt:taboption("advanced", Value, "will_qos", translate("Will QoS"))
will_qos.datatype = "range(0,2)"

will_retain = section_mqtt:taboption("advanced", Flag, "will_retain", "Will Retain", translate("Should will message be retained"))
will_retain.default = "0"
will_retain.rmempty = false

reconnect_delay = section_mqtt:taboption("advanced", Value, "reconnect_delay", "Reconnect Delay", translate("Number of seconds to wait before retrying to connect broken connection"))
reconnect_delay.datatype = "range(1,900)"
reconnect_delay.default = 5

reconnect_delay_max = section_mqtt:taboption("advanced", Value, "reconnect_delay_max", "Reconnect Delay Max", translate("Maximum number of seconds to wait before retrying"))
reconnect_delay_max.datatype = "range(1,3600)"
reconnect_delay_max.default = 900

reconnect_exponential_backoff = section_mqtt:taboption("advanced", ListValue, "reconnect_exponential_backoff", "Use Exponential Backoff", translate("Reconnect Exponential Backoff"))
reconnect_exponential_backoff:value("1", translate("Yes"))
reconnect_exponential_backoff:value("0", translate("No"))
reconnect_exponential_backoff.default = "0"

max_inflight_messages = section_mqtt:taboption("advanced", Value, "max_inflight_messages", "Maximum Inflight Messages", translate("Maximum In-flight messages"))
max_inflight_messages.datatype = "range(1,100)"
max_inflight_messages.default = 20

return m